/* Refer to http://webtop.us.dell.com/webtop/drl/objectId/0b014781809fd8cb  for Details on the IPMI Commands Definitions */

#ifndef _CM_IPMI_LIB_H
#define _CM_IPMI_LIB_H
#include <sys/types.h>
#include <iostream>
#include <cassert> // For assert().
#include <string>
//#include <pthread.h>

// using std;
using std::string;

#ifndef _STDINT_H
typedef unsigned char  uint8_t;
typedef unsigned short uint16_t;
typedef unsigned int   uint32_t;
typedef unsigned long  uint64_t;
#endif // _STDINT_H

// IPMI Request and Response message lengths
const uint16_t IPMI_HDRSIZE = 2;
const uint16_t BEGINMASERSESS_REQLEN = 6 + IPMI_HDRSIZE;
const uint16_t BEGINMASERSESS_RSPLEN = 7 + IPMI_HDRSIZE;
const uint16_t ENDMASERSESS_REQLEN = 5 + IPMI_HDRSIZE;
const uint16_t ENDMASERSESS_RSPLEN = 3 + IPMI_HDRSIZE;
const uint16_t MASERSESSWDOG_REQLEN = 5 + IPMI_HDRSIZE;
const uint16_t MASERSESSWDOG_RSPLEN = 5 + IPMI_HDRSIZE;
const uint16_t GETPARTINDEX_REQLEN = 5 + IPMI_HDRSIZE;
const uint16_t GETPARTINDEX_RSPLEN = 11 + IPMI_HDRSIZE;
const uint16_t GETPARTINFO_REQLEN = 7 + IPMI_HDRSIZE;
const uint16_t GETPARTINFO_RSPLEN = 12 + IPMI_HDRSIZE;
const uint16_t ATTACHPART_REQLEN = 11 + IPMI_HDRSIZE;
const uint16_t ATTACHPART_RSPLEN = 5 + IPMI_HDRSIZE;
const uint16_t GETMASERSTATUS_REQLEN = 6 + IPMI_HDRSIZE;
const uint16_t GETMASERSTATUS_RSPLEN = 5 + IPMI_HDRSIZE;
const uint16_t DETACHPART_REQLEN = 11 + IPMI_HDRSIZE;
const uint16_t DETACHPART_RSPLEN = 5 + IPMI_HDRSIZE;
const uint16_t CREATEDYNPART_REQLEN = 12 + IPMI_HDRSIZE;
const uint16_t CREATEDYNPART_RSPLEN = 7 + IPMI_HDRSIZE;
const uint16_t DELETEDYNPART_REQLEN = 9 + IPMI_HDRSIZE;
const uint16_t DELETEDYNPART_RSPLEN = 5 + IPMI_HDRSIZE;
const uint16_t DETACHDYNPART_REQLEN = 9 + IPMI_HDRSIZE;
const uint16_t DETACHDYNPART_RSPLEN = 5 + IPMI_HDRSIZE;
const uint16_t UPDATEHASHHDR_REQLEN = 11 + IPMI_HDRSIZE;
const uint16_t UPDATEHASHHDR_RSPLEN = 5 + IPMI_HDRSIZE;
const uint16_t SECMASERUP_HASHDATA_REQLEN = 15 + IPMI_HDRSIZE;
const uint16_t SECMASERUP_HASHDATA_RSPLEN = 5 + IPMI_HDRSIZE;
const uint16_t CMDREGSSIBTASK1_REQLEN = 6 + IPMI_HDRSIZE;
const uint16_t CMDREGSSIBTASK1_RSPLEN = 5 + IPMI_HDRSIZE;
const uint16_t CMDREGSSIBTASK2_REQLEN = 15 + IPMI_HDRSIZE;
const uint16_t CMDREGSSIBTASK2_RSPLEN = 5 + IPMI_HDRSIZE;
const uint16_t GETMASERTYPE_REQLEN = 2 + IPMI_HDRSIZE;
const uint16_t GETMASERTYPE_RSPLEN = 4 + IPMI_HDRSIZE;
const uint16_t GETMASERACCESSSTATE_REQLEN = 2 + IPMI_HDRSIZE;
const uint16_t GETMASERACCESSSTATE_RSPLEN = 4 + IPMI_HDRSIZE;
const uint16_t SETMASERACCESSSTATE_REQLEN = 4 + IPMI_HDRSIZE;
const uint16_t SETMASERACCESSSTATE_RSPLEN = 3 + IPMI_HDRSIZE;
const uint16_t QUERYLCL_REQLEN = 5 + IPMI_HDRSIZE;
const uint16_t QUERYLCL_RSPLEN = 5 + IPMI_HDRSIZE;
const uint16_t GETFLAGSTATUS_REQLEN = 6 + IPMI_HDRSIZE;
const uint16_t GETFLAGSTATUS_RSPLEN = 6 + IPMI_HDRSIZE;
const uint16_t SETFLAGSTATUS_REQLEN = 5 + IPMI_HDRSIZE;
const uint16_t SETFLAGSTATUS_RSPLEN = 5 + IPMI_HDRSIZE;
const uint16_t GETPMSTATUS_REQLEN = 6 + IPMI_HDRSIZE;
const uint16_t GETPMSTATUS_RSPLEN = 32 + IPMI_HDRSIZE;

const uint16_t RESEXTCFG_REQLEN = 3 + IPMI_HDRSIZE;
const uint16_t RESEXTCFG_RSPLEN = 5 + IPMI_HDRSIZE;
const uint16_t SETEXTCFG_REQLEN = 15 + IPMI_HDRSIZE;
const uint16_t SETEXTCFG_RSPLEN = 5 + IPMI_HDRSIZE;

const uint16_t STAGEANDUPDATE_REQLEN = 5+IPMI_HDRSIZE;
const uint16_t STAGEANDUPDATE_RSPLEN = 6+IPMI_HDRSIZE;

const uint16_t GETIDRACVER_REQLEN = 1 + IPMI_HDRSIZE;
const uint16_t GETIDRACVER_RSPLEN = 21 + IPMI_HDRSIZE;

const uint8_t FLOPPY	= 0x00;
const uint8_t RESERVED	= 0x20;
const uint8_t USB_KEY	= 0x40;
const uint8_t READONLY  = 0x00;
const uint8_t READWRITE = 0x08;

typedef enum eOEMIPMIRequestType {POSTOnlyMaserCmd	= 0xA1,
								MaserSessionCommand	= 0xA2,
								GetMaserInfo		= 0xAB,
								RestoreMaserImage	= 0xAC,
								GetMASERType		= 0xAD,
								GetMASERAccessState	= 0xAE,
								SetMASERAccessState	= 0xAF,
								ReservedExtCfgCmd	= 0x01,
								SetExtCfgCmd		= 0x03,
								Resetidrac          = 0x02,
								PMStatusCmd         = 0xA9,
								ManufacturingMode   = 0xA5,
								GetSystemConf       = 0x59,
								Get13GiDRACVersion       = 0xBF,


}OEMIPMIRequestType;
typedef enum idracSubCommands {
// Sesison commands
	
	BeginMASERSession        =0x00,
    EndMASERSession          =0x01,
    ResetMASERSessionWD      =0x02,
    Getpartitionindexinfo    =0x03,
    GetPartitionInfo         =0x04,
    AttachPartitions         =0x05,
    DetachPartitions         =0x06,
    CreateDynamicPartition   =0x07,
    DeleteDynamicPartition   =0x08,
    SecureMASERUpdate        =0x09,
    GetMASERStatus           =0x0A,
    ChangePartitionAccessType=0x0B,
    RegisterSSIBTask         =0x0C,
	getStateFlagCmd			 =0x10,
	setStateFlagCmd			 =0x11,
	BeginMASERSessionExtended=0x12,
	BeginSecureUpdate		 =0x20,
	ProcessSecureUpdate		 =0x22,
	EndSecureUpdate			 =0x23,
	StageAndUpdate           =0x25
} idracCommand;

typedef enum lclSubCommands {
// Sesison commands
    ReservedUndefined_0		=0x00,
    updateXMLRecords		=0x01,
    updateEntireInventory   =0x02,
	resUndef_03_10			=0x0A,
    queryCurrentUsingXML	=0x0B,
    queryHistoryUsingXML    =0x0C,
    queryEventRecords		=0x0D,
	resUndef_14_20			=0x14,
	getLCLStatus			=0x15,
	resUndef_22_255			=0xff
} lclCommand;

typedef enum ipmiPMStatusSubCommands{
	 GetPMStatusValue         =0x12,
	 GetBrandNameValue        =0x14,
	 GetResellerID            =0xF7,
	 SetPMInstall             =0x15
} pmCommands;

struct ipmiRequest {
        struct {
                uint8_t netFn:6;
                uint8_t lun:2;
                uint8_t cmd;
                uint8_t target_cmd;
                uint16_t data_len;
                uint8_t *data;
        } Msg;
};
typedef enum flagTypes { unkFlag=0, fDebug, fVerbose, fLogging ,fSimulation} flagType;
typedef enum esessionState { unkState=0, Stale, Active, Inactive} sessionState;
struct rtiFlags {
    uint8_t Verbose:1;
    uint8_t Debug:1;
    uint8_t Logging:1;
    uint8_t Simulation:1;
};    
typedef struct {
    sessionState Session;
    uint16_t     MaserHandle;      // 2 byte value returned from creating a session.
    uint16_t     InitialTimeout;   // Timeout value set at initialization time
    bool         CompletionCode;   // Completion code of last request
    uint8_t      partBitmap;       // Bitmap of partitions presetn on the Maser

} ipmiHandle;

typedef enum UEFI_StateFlagsValues {
    STATE_FLAG_NORMAL_BOOT		= 0x00, //Normal Boot
    STATE_FLAG_GUI				= 0x01, //GUI
    STATE_FLAG_RESERVED2		= 0x02, //Reserved
	STATE_FLAG_RESERVED3		= 0x03, //Reserved
    STATE_FLAG_FIELD_SERVICE	= 0x04, //Field Service
    STATE_FLAG_SSM				= 0x05, //SSM
    STATE_FLAG_OVERRIDE			= 0x80, //Override Existing session (Normal boot)
} UEFI_StateFlags;

typedef  struct sResponseBuffer {
    uint8_t BufferByte[8];
} ipmiResponseBuffer;

// Generic Header for all IPMI OEM commands
#define MASER_OEM_IPMI_HDR \
	uint8_t MaserNetFun; \
	uint8_t MaserOEMCmd;

#pragma pack(push,1)

//Check Doc
typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t   subCmd;
   uint16_t  maserHdl;
   uint16_t  szInChunks;
   uint8_t   volName[6];
   uint8_t   acDevType;
   } CreateDynPartReq;

typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t cmpCode;
   uint16_t maserHdl;
   uint16_t dynPartBMap;
   uint16_t reserved;
   } CreateDynPartRsp;
//Check Doc

//Refer Table 4 Begin MASER Session in the IPMI Commands Doc
typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t   subCmd;
   uint8_t   reserved1;
   uint16_t  wDogTimer;
   uint16_t  reserved2;
   } BeginMaserSessReq; 

//Refer Table 5 End MASER Session in the IPMI Commands Doc
typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t   subCmd;
   uint16_t  maserHdl;
	uint16_t  reserved;
   } EndMaserSessReq; 

typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t   cmpCode;
   uint16_t  maserHdl;
   } GenericMaserRsp;

//Refer Table 11 Delete Dynamic Partition in the IPMI Commands Doc
typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t   subCmd;
   uint16_t  maserHdl;
   uint16_t  dynPartBMap;
   uint8_t  reserved[4];
   } DelDynPartReq;

//Refer Table 9 Detach Partitions in the IPMI Commands Doc
typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t   subCmd;
   uint16_t  maserHdl;
   uint16_t  statPartBMap;
   uint16_t  reserved1;
   uint16_t  dynPartBMap;
   uint16_t  reserved2;
   } DetMaserPartReq;

//Refer  Table 8 Attach Partitions in the IPMI Commands Doc
typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t   subCmd;
   uint16_t  maserHdl;
   uint16_t  statPartBMap;
   uint16_t  reserved1;
   uint16_t  dynPartBMap;
   uint16_t  reserved2;
   } AtMaserPartReq;

//Refer  Table 16 Change Partition Access Type in the IPMI Commands Doc
typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t   subCmd;
   uint16_t  maserHdl;
   uint16_t  statPartBMap;
   uint16_t  reserved1;
   uint16_t  dynPartBMap;
   uint16_t  reserved2;
   uint8_t   accessType; // read - 0 readwrite 1
   uint16_t  reserved3;   
   } ChgPartAccModeReq;

//Refer  Table 13 Secure Update Header in the IPMI Commands Doc
typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t   subCmd;
   uint16_t  maserHdl;
   uint8_t   headerRData;
   uint16_t  dynPartBMap;
   uint8_t   hashType;
   uint16_t  hashCksum;
   uint16_t  reserved;
   } secureMaserHeaderUpdate;

//Refer  Table 14 Secure Update Data in the IPMI Commands Doc
typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t   subCmd;
   uint16_t  maserHdl;
   uint8_t   headerRData;
   uint8_t   hashData[10];
   uint8_t   reserved;   
   } secureMaserDataUpdate;

//Refer  Table 15 GetStatus of MASER IPMI subcommand in the IPMI Commands Doc 
typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t   subCmd;
   uint16_t  maserHdl;
   uint8_t   oldSubCmd;
   uint16_t   reserved;   
   } GetMaserStatus;

//Refer  Table 15 GetStatus of MASER IPMI subcommand in the IPMI Commands Doc 
typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t   cmpCode;
   uint16_t  maserHdl;
   uint8_t   subcmdstat;
   uint8_t   reserved;   
   } GetMaserStatusRsp;

//Refer  Table 7 Get Partition Index Info in the IPMI Commands Doc - Request
typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t   subCmd;
   uint16_t  maserHdl;
   uint16_t  reserved2;
   } GetPartIndexReq; 
//Refer  Table 7 Get Partition Index Info in the IPMI Commands Doc - Response
typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t   cmpCode;
   uint16_t  maserHdl;
   uint16_t  statPartBMap;
   uint16_t  reserved1;
   uint16_t  dynPartBMap;
   uint16_t  reserved2;
   } GetPartIndexRsp;
const uint8_t	staticPartType = 0;
const uint8_t	dynamicPartType = 0x80;

//Refer Table 8  Get Partition Info in the IPMI Commands Doc - Request
typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t   subCmd;
   uint16_t  maserHdl;
   uint8_t   partType;
   uint8_t   partIndex;
   uint16_t  reserved1;
   } GetPartInfoReq; 
//Refer Table 8  Get Partition Info in the IPMI Commands Doc - Response
typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t   cmpCode;
   uint16_t  maserHdl;
   uint16_t  partChunks;
   uint8_t   partId[6];
   uint8_t   partStat;
   } GetPartInfoRsp;

//Refer Table 30 Get MASER Type in the IPMI Commands Doc - Request
typedef struct {
   MASER_OEM_IPMI_HDR
   uint16_t  res_1;
   } GetMaserTypeReq; 
//Refer Table 30 Get MASER Type in the IPMI Commands Doc - Response
typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t	cmpCode;
   uint8_t	maserTyp; // 0 = Full MASER 1=MASER Lite
   uint16_t	res_1;
   } GetMaserTypeRsp;

//Refer Table 28 Get MASER Access State in the IPMI Commands Doc - Request
typedef struct {
   MASER_OEM_IPMI_HDR
   uint16_t  res_1;
   } GetMaserAccessStateReq; 
//Refer Table 28 Get MASER Access State in the IPMI Commands Doc - Response
typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t	cmpCode;
   uint8_t	maserAccess; // 0 = Enabled 1=Disabled
   uint16_t reserved;
   } GetMaserAccessStateRsp;

//Refer Table 29 Set MASER Access State in the IPMI Commands Doc - Request
typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t   subCmd;
   uint16_t  res_1;
   } SetMaserAccessStateReq; 
//Refer Table 29 Set MASER Access State in the IPMI Commands Doc - Response
typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t	cmpCode;
   //uint8_t	newState; // 0 = Enabled 1=Disabled //Doc doesnt define this byte *****************************************
   uint16_t	res_1;
   } SetMaserAccessStateRsp;

typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t  subCmd;		//byte 1
   uint16_t  maserHdl;	//byte 2-3	
   uint8_t  flagType;	//byte 4  (0=UEFI, 1=Recovery, 2+=res)
   uint16_t	res_1;		//byte 5-6
   } getStateFlagReq;

typedef struct { 
	MASER_OEM_IPMI_HDR
	uint8_t subCmd; //byte 1
	uint16_t maserHdl; //byte 2-3
	uint8_t flagType; // 0x01= Check Comaptibility; 0x02= StageOnly 0x04 StageAndUpdate 0x08=status
	uint8_t reserve1;
} doStageAndUpdateReq;

typedef struct { 
	MASER_OEM_IPMI_HDR
	uint8_t cmpCode; //byte 1
	uint16_t maserHdl; //byte 2-3
	uint8_t cmdResp; // 0x01= Check Comaptibility; 0x02= StageOnly 0x04 StageAndUpdate 0x08=status
	uint16_t reserve;
} doStageAndUpdateRsp;


typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t	cmpCode;	//byte 1
   uint16_t maserHdl;	//byte 2-3
   uint8_t	flagValue;	//byte 4 For UEFI 0=Normal Boot,1=GUI,2=res, 3=res, 4=FieldService,5=SSM,80=override session(normal boot)
						//       For Recovery 0=Not in recovery, 1=in recovery, 2+=res
   uint16_t	res_1;		//byte 5-6
   } getStateFlagRsp;
typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t  subCmd;		//byte 1
   uint16_t  maserHdl;	//byte 2-3	
   uint8_t  flagType;	//byte 4  (0=UEFI, 1=Recovery, 2+=res)
   uint8_t  flagValue;   //byte 5  For UEFI 0=Normal Boot,1=GUI,2=res, 3=res, 4=FieldService,5=SSM,80=override session(normal boot)
						//		  For Recovery 0=Not in recovery, 1=in recovery, 2+=res
   } setStateFlagReq;

typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t	cmpCode;	//byte 1
   uint16_t maserHdl;	//byte 2-3
   uint16_t	res_1;		//byte 4-5
   } setStateFlagRsp;

typedef struct{
	MASER_OEM_IPMI_HDR
	uint8_t IANA_1;				//0xa2
	uint8_t IANA_2;				//0x02
	uint8_t IANA_3;				//0x00
} ReservedExtCfgReq;

typedef struct{
	uint8_t cmpCode;			//0x00
	uint8_t IANA_1;				//0xa2
	uint8_t IANA_2;				//0x02
	uint8_t IANA_3;				//0x00
	uint8_t res_id;
} ReservedExtCfgRsp;

typedef struct{
	MASER_OEM_IPMI_HDR
	uint8_t IANA_1;				//0xa2
	uint8_t IANA_2;				//0x02
	uint8_t IANA_3;				//0x00
	uint8_t res_id;				//Get it from the ReservedExtCfgRsp - res_id
	uint8_t token_id;			//Should be 0x14 for Graceful Reset;
	uint8_t index;				//0x00
	uint8_t dataOffset_LSB;		//0x00
	uint8_t dataOffset_MSB;		//0x00
	uint8_t inProgress;			//0x01
	uint8_t token_data1;		//0x06	
	uint8_t token_data2;		//0x00
	uint8_t token_data3;		//0x01
	uint8_t token_data4;		//0x01
	uint8_t token_data5;		//0x00
	uint8_t token_data6;		//0x01
} SetExtCfgReq;

typedef struct{
	uint8_t cmpCode;			
	uint8_t IANA_1;				
	uint8_t IANA_2;				
	uint8_t IANA_3;				
	uint8_t res;	
} SetExtCfgRsp;

typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t   versionInfo;
   
   } GetiDracVerReq; 

typedef struct {
   MASER_OEM_IPMI_HDR
   uint8_t   compcode;
   uint8_t   MajorFrmVer;
   uint8_t   MinorFrmVer;
   uint16_t  BuildNum;
   uint16_t  PlatformDataVer;
   uint16_t  LCVersion;
   uint16_t  LCBuild;
   uint8_t   reserved[10];
   } GetiDracVerRsp; 

#pragma pack(pop)

#endif // _CM_IPMI_LIB_H
