//--------------------------------------------------------------------
//
//          DELL COMPUTER CORPORATION PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Computer Corporation and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-2002 Dell Computer Corporation. All Rights Reserved.
// 
//
// Abstract/Purpose:
//
// Header file for basic data structures and macros.
//
// Environment:
//
// WINNT, Netware and OS2
//
//---------------------------------------------------------------------
#ifndef MASER_H
#define MASER_H

#include <cm_types.h>
#include <ctime>	// time() for rand()
#include <cstdlib>
#include <cstring>	// strcmp()
#include <fstream>
#include <iostream>
#include <iomanip>
#include <vector>
#include <cm_ipmilib.h>
//#include <dir.h>	// directory handling
//#include <dir>

//#define SYSTEM_TYPE_WINNT

#if defined SYSTEM_TYPE_WINNT
#if !defined(_WIN32_WINNT)
#define  _WIN32_WINNT   0x0500
#endif
#include <windows.h>
#include <winioctl.h>
#include <tchar.h>
#include <io.h>
#include <Setupapi.h>
#include <cfgmgr32.h>

#elif defined SYSTEM_TYPE_CYGWIN
#include <windows.h>  // For the ULONG types...
#include <dlfcn.h>
#elif defined SYSTEM_TYPE_LINUX
#define MAX_PATH (1024)
#include <sys/time.h>
#include <unistd.h>
#include <sys/times.h>
#include <dirent.h>
#include <sys/stat.h>
#endif

// dont move the position of the following .h files
// they are position dependent!!

#include <dcvertag.h>
#include <dcpcidef.h>
#include <dcdetypes.h>
#include <dcapidef.h>
#include <esmapi.h>
#include <esmiocts.h>
#include <dcsmstatus.h>
#include <dchipmapi.h>
#include <dcdetypes.h>

#ifndef _STDINT_H
typedef unsigned char  uint8_t;
typedef unsigned short uint16_t;
typedef unsigned int   uint32_t;
typedef unsigned long  uint64_t;
#endif // _STDINT_H


typedef unsigned short maserTime_t;
typedef _EsmIPMICmdIoctlReq  IPMIBuf;
#define IPMI_BUFFER(tipe) (tipe *) IPMIReqBuf.Parameters.IRREx.ReqRspBuffer

// IPMI OEM message constants
const uint8_t MASER_BMC_ADD      = 0x20;
const uint8_t MASER_NETFUN_LUN   = 0xc0;
#ifdef IDRAC_P06
const uint8_t MASER_IPMI_OEM_CMD = 0xa0;
#else
const uint8_t MASER_IPMI_OEM_CMD = 0xa2;
#endif

// IPMI OEM Completion codes
const uint8_t MASER_SUCCESS				= 0x00;
const uint8_t MASER_FAILURE				= 0x01;
const uint8_t MASER_PENDING				= 0x02;
const uint8_t MASER_INVALID				= 0x03;
const uint8_t MASER_OVERFLOW			= 0x04;
const uint8_t MASER_DISABLED			= 0x05;
const uint8_t MASER_BMAP_IMAGE_INVALID	= 0x06;
const uint8_t MASER_SESSION_IN_PROGRESS	= 0x07;
const uint8_t MASER_PARTITIONS_ATTACHED	= 0x08;
// Sometimes the Create Dynamic partition returns 0xff
// after or while it is creating the dynamic partition.
// We are going to treat 0xff as pending.
const uint8_t MASER_INPROCESS			= 0xff;

// Some Convenient Values
const int		SMAL_MAXPOLLCNT			= 3000;			// Seconds or 9 minutes
const int		MASER_POLL_TIMEOUT		= 500;			// Seconds 8.3 minutes
const int		MASER_SESSION_TIMEOUT	= 1200;			// Seconds 20 minutes
const uint16_t	DYNCHUNK_SZ				= 0x8000;		// Bytes 32768
const uint32_t	MAXCHUNK_SZ				= 0x80000000;	// Bytes 2147483648
const uint32_t	ONE_MEG					= 1048576;		// Bytes
const uint32_t THIRTYTWO_MEG    				= 32*1024*1024;
const uint32_t	TWO_MEG					= 2097152;		// Bytes
const uint32_t	ONE_K					= 1024;			// Bytes
const uint16_t	emptyBitMap				= 0;
const int		ENOERROR				= 0;
const uint32_t	MAX_STAT_COUNT			= 120;
// timeout in milliseconds
const int     IPMI_TIMEOUT				= 90;

static const int MAXBSIZE = 25 * 1024 * 1024;       /* 25 MB */

extern uint8_t OEMIPMIRequest(	OEMIPMIRequestType	cmd,
								uint8_t				subCmd,
								IPMIBuf				*IPMIReqBuf,
								uint16_t			reqBufSz,
								uint16_t			rspBufSz);

extern uint8_t OEMIPMIRequestiDRACReset(OEMIPMIRequestType	cmd, 
										 IPMIBuf			*IPMIReqBuf, 
										 uint16_t			reqBufSz,
										 uint16_t			rspBufSz);

//extern bool PollMaserCmdComplete(int PollCount,uint8_t PollCmd);
extern int InstallIPMI();
extern int UnInstallIPMI();
extern int FreeIPMILibrary();
extern void* GetFunctPtr (const char *lpProcName);
extern bool	DebugLog;
extern bool	BenchFlag;
extern const char *dynDriveName;
extern string RHatPartDir;
extern string RHat4DRACDev;
extern string RHat5DRACDev;
extern string RHat4DRACStatDev;
extern string RHat5DRACStatDev;
extern string RHat5DRACStatDevPt1;
extern string RHat5DRACStatDevPt2;
extern string DynSecMasDev;
extern string StatSecMasDev;
extern string RedHatRel;
extern string SuseRel;
extern string ESXRel;
extern string DTKRel;
extern string CDURel;
extern string NewCDURel;
// extern const int REDHAT4;
// extern const int REDHAT5;
// extern const int REDHAT6;
// extern const int SUSE9;
// extern const int SUSE10;
// extern const int SUSE11;
extern const int REDHAT;
extern const int SUSE;
extern const int XENSERVER;
extern const int ELI;
extern const int ELI_NEW;
extern const int ESX;
extern const int ESX4;
extern const int CENTOS;
extern const int ESX4_1;
extern const int ESXi;
extern const uint8_t UMOUNT_WAIT_TIME;
extern const string sLink;

extern void logMaserMsg(const char *file,unsigned int line,const string str);
// SMAL internal error code
const uint8_t SMAL_MAX_THROW_MSG = 0xfe; // This is as big as they get
#include <smal.h>
extern uint16_t dynHandleIdx;
const int	MaxPartInfoEntry = 257;
extern partInfo maserPartInfo[MaxPartInfoEntry];

extern bool CreateLCLConfigFile(ostringstream *Stage,vector<cfgEntry> queryRequest);
//extern partInfo maserDynpartInfo[MaxPartInfoEntry];

#ifdef SYSTEM_TYPE_WINNT
#define unlink _unlink
inline const unsigned int cm_sleep(unsigned int seconds) {
   Sleep(seconds * 1000);
   return seconds;
}
DEVINST GetDrivesDevInstByDeviceNumber(long DeviceNumber, UINT DriveType, char* szDosDeviceName);
int UsbSafeRemove(char DriveLetter);
#endif
#ifdef SYSTEM_TYPE_LINUX
inline char *strcpy_s(char *dest, size_t n, const char *src){
        char *strPtr;
        strPtr = strncpy(dest, src, n);
        return strPtr;
}
inline char *strcat_s(char *dest, size_t n, const char *src){
        char *strPtr;
        strPtr = strncat(dest, src, n);
        return strPtr;
}
inline const unsigned int cm_sleep(unsigned int seconds) {
   sleep(seconds);
   return seconds;
}
#define sprintf_s snprintf
typedef int errno_t;

#endif
#endif
