//--------------------------------------------------------------------
//
//		  DELL COMPUTER CORPORATION PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with Dell Computer Corporation and may not
// be copied or disclosed except in accordance with the terms of that
// agreement.
//
// Copyright (c) 1995-2008 Dell Computer Corporation. All Rights Reserved.
// 
//
// Abstract/Purpose:
//
// Secure Maser Access User Include file
//
// Environment:
//
// WINNT, Netware and OS2
//
//---------------------------------------------------------------------
#ifndef SMAL_H
#define SMAL_H

#if defined SYSTEM_TYPE_WINNT
	#include <libxml\\xmlmemory.h>
	#include <libxml\\parser.h>
	#include <libxml\\xpath.h>
	#include <Shlobj.h>

#elif SYSTEM_TYPE_LINUX
	#include <libxml/xmlmemory.h>
	#include <libxml/parser.h>
	#include <libxml/xpath.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <iostream>
#include <sstream>
#include <vector>
#include <iomanip>
#include <smal_error.h>

#if defined WIN32  || defined WIN64
#include <Windows.h>
#endif 

using std::string;
using std::cout;
using std::stringstream;
using std::ostringstream;
using std::ifstream;
using std::ofstream;
using std::vector;
using std::ios;
using std::endl;
using std::stringstream;
using std::right;
using std::hex;
using std::dec;
using std::setw;

#ifndef _STDINT_H
typedef unsigned char  uint8_t;
typedef unsigned short uint16_t;
typedef unsigned int   uint32_t;
typedef unsigned long  uint64_t;
#endif // _STDINT_H

//typedef unsigned short partId_t;
typedef unsigned short maserTime_t;
typedef enum epartMode {readMode=0, readWriteMode, unknownMode } partMode_t;
typedef enum exmlMode {
	doNotDisturb=0,filecopy,dircopy,filemove,dirmove,filedelete,dirdelete,fileappend,dirmake,ssib,
	lcl_update_records,lcl_update_entire_inventory,lcl_query_current_records,
	lcl_query_history_records, lcl_query_event_records, lcl_query_dependency, signedunzip
} xmlMode_t;

typedef enum {eSuccess=0,objectInitializationFailed,newSessionFailed,noDestinationFile,noSourceFile,SrcFileMissing,addOrderError, eLastEc} errCode_t;
typedef enum {Failure=-1,Pre12G=0, TwelveG=1, ThirteenG =2} generation_t;

const static uint16_t MAXPATH_LEN = 360;

// definitions to map a maser partition to its
// bitmap equivalent. The defined partitions
// on the iDRAC are commented here.
typedef uint16_t partId_t;
const uint16_t maxPartitions = 257;
const uint16_t dynPartSlot = 0;

typedef enum epartFormat {formatInvalid=0, FAT32, FAT16, Ext2, UnknownPart } partFormat;
typedef enum emaserSupport {noMaserPresent=0,fullMaserPresent,MaserLite,Liter} maserSupport_t;
typedef enum epartType { partInvalid=0, StaticPart, DynamicPart } partType; 
typedef enum eIdentityModuleStatus { Error=-1, DEFAULT=0, DEBRAND=1, REBRAND=2, STRINGREBRAND=3} eIdentityModuleStatus;
typedef struct serrorInfoStr {
   uint8_t lastErr;
   uint8_t errSubcode;
   } errorInfoStr;
/*|Partition|Partition|Partition|Max |Volume|Partition|Bootable/|Platform|Present|Max Space|Used Space|Available|
  |	ID   |  Image  |   Type  |Size| Label|  file   |Thunking |		|	   |		 |		  |  Space  |
  |		 |  Name   |		 |(mb)|	  | Format  |Supported|		|	   |		 |		  |		 |*/
typedef struct spartInfo {
	partId_t		i_partid;
	string			i_image_name;
	partType		i_partType;	//Fixed/dynamic
	uint16_t		i_mbSize;
	string			i_volumeLabel;
	partFormat		i_partFormat;  //FATxx etc.
	bool			i_bootable_thunking_sup;
	string			i_platform;
	string			i_mountPoint;
	uint16_t		i_bitmap;
	bool			i_present;
	uint64_t		i_space;
	uint64_t		i_used;
	uint64_t		i_avail;
	errorInfoStr	i_errorInfo;
} partInfo;


typedef struct {
	uint16_t		MASER_Handle;
	uint16_t		DYNMASER_Handle;
	bool			sessionActive;
	uint16_t		statBitMap;
	uint16_t		dynBitMap;
	maserTime_t		wdTimeOut;
}sessionHandle;


typedef struct {
	int			order;
	xmlMode_t	mode;
	int			algorithm;
	int			copyRc;
	int 		filePerms;
	off_t		fileSize;
	int			n_or_n1;	// 0 = no n processing
							// 1 = this file goes to n
							// 2 = this file goes to n-1
	off_t	  loc;
	uint16_t	partNum;
	uint16_t	processed;
	string		srcName;
	string		dstName;
	string		hash;
}cfgEntry;


int syncAll(string mntPnt);

class CUpdateLCLTest;
class CNotificationThread;

class secureMaser 
{
	friend class CUpdateLCLTest;
	errCode_t			sessionEc;
	sessionHandle		thisSession;
	vector<cfgEntry>	FileList;
	int					l_state;
	errorInfoStr		errorInfo;
	maserSupport_t		MASERType;	// 0 = No Maser Present
									// 1 = Full Maser
									// 2 = Maser Lite
									// 3 = Liter
	int					l_entryNumber;
	string				mountedVolName;
	int			curLCDRIVECnt,curStatCnt, curDynCnt;

	// 'LinuxSystem keeps track of what Linux system is booted.
    // RedHat4=0, RedHat5=1,SuSe9=2,SuSE10=3
	int LinuxOSVer;
	int restart;
#if defined WIN32  || defined WIN64
	CNotificationThread * m_pNotificationThread;
	std::string m_sNewMappedDriveLetter;
	std::string m_sNewMappedDriveLabel;
#endif 	
	int n_payload_added;
	string n_payload_component_id;
	uint32_t MAX_DYN_PART_SIZE;

	bool isFreshInstall;
#ifdef SYSTEM_TYPE_LINUX
	bool m_bHaldDaemonRunning;
#endif

public:
	bool		InitOk;
				secureMaser();
	virtual		~secureMaser();
	bool		AttachMaserPartition(string PartName);	
	bool		DetachMaserPartition(string PartName);
	bool		DetachAll(void);
	bool		StartCopyFile(void);
	bool		EndCopyFile(void);
	bool		AddFile (string src, string dest, exmlMode action);
	bool		AddFile (string src, string dest, bool SplitFile);
	bool		AddNFile (string file);
	bool		AddN1File (vector<cfgEntry> n1List);
	bool		DeleteMaserFile(string whichOne);
	bool		CompleteCopyFile(void);
	void		CalculateAndAdjustPayloadSize(uint32_t &payloadSize);
	bool		CompleteSignedUnzip(void);
	bool  populateDynPart(string mntPnt, ostringstream *xmlStr, size_t &start, size_t dropBoxSize);
	bool  populateDynPart13G(string mntPnt, ostringstream *xmlStr, size_t &start, size_t dropBoxSize);
	bool		UpdateLCL(string cid, xmlMode_t updateType, string xmlStr = "");
	bool		QueryLCL(stringstream &buffer, xmlMode_t queryType );
	bool		doDepCheck(stringstream &buffer, xmlMode_t queryType );
	bool		getPartDrive (const string volName, string *mountPoint);
	bool		IsMaserEnabled(void);
	bool		getPartInfo (uint16_t partNum);
	bool		getDynPartInfo (uint16_t partNum);
	uint8_t		getSMALError();
	uint16_t	getPartIndexInfo (void);
	uint16_t	getDynPartIndexInfo (void);
	bool		getLinuxOSVer();
	maserSupport_t IsMaserPresent(void);
	bool		secureMaserInit(void);
	uint8_t     getStateFlag(uint8_t &Status);
	uint8_t		setStateFlag(const uint8_t Status);

	int			regPowerCycle();
	int			getPSCount(int &numPSU); /* PSU DUP */
	int			getPSUVersion(uint8_t i, unsigned int &compID, string &version, unsigned int &isPresent); /* PSU DUP */

	bool		ResetiDRAC(maserTime_t SessionTimeout);
	bool		UpdateCompletionStatus; //Completion status set for UEFI state flag.
	void		setRestart();
	bool		get_comp_id(string &comp_id);
	bool        getGuid(string &guid);
	xmlXPathObjectPtr GetNodeSet(xmlDocPtr pXmlDoc, xmlChar *pXpathExpression);
	generation_t GetSystemGeneration();
	bool		AttachAndCompare(bool & compResult);
	bool		GetFreshInstall() { return isFreshInstall; }
    eIdentityModuleStatus getIPMIPMStatus();
	string      getPMVersion();
	bool		isPMInstalled();
	string      getResellerId();
	string      getManufacturingMode();
	string      getServiceTag();
	bool        setPMInstall();
	bool        IsPayloadSigned (string &m_strPayloadFileName);
	bool		CompleteCopyFile13G();
	bool		checkGUIDEnable();
	int			sendIPMICommandToStageAndUpdate(bool bStageAndUpdate);
	int			pollForStatus();
	int			iS13GiDRAC(void);
	bool        isESXiOS();
protected:
	bool	DeleteDropBox( uint16_t iBitMap=0);
	bool	DetachPartition(uint16_t statBitMap, uint16_t dynBitMap,partId_t partId = 0, bool bDetachAll= false);
private:
	bool		getPartDrive2 (const string volName, string *mountPoint);
	bool		BeginMaserSession(maserTime_t SessionTimeout);
	bool		BeginMaserSessionExtended(maserTime_t SessionTimeout);
	bool		secureMaserInitIPMI(void);
	bool		EndMaserSession(void);
	bool		AttachMaserPartition(partId_t partId);
	bool		DetachMaserPartition(partId_t partId);
	bool		IsSessionActive(void);
	uint32_t	CalculateFreeMaserSpace(partId_t partId, vector<string> Filenames);
	void		LxGetVolNameInfo(partId_t partId, const char *VolName);
	
	bool	CreateDropBox(const char *volName, uint32_t size,uint32_t *newSize);
	
	bool	AttachPartition(uint16_t statBitMap, uint16_t dynBitMap,partId_t partId = 0);

	
	void	ESX_RemoveEntries(uint16_t statBitMap, uint16_t dynBitMap,partId_t partId = 0);

	bool	getDynPartDrive (const char *volName, char *mountPoint);
	bool	getDynPartDrive (const string volName, string *mountPoint);
	bool	deleteDynPartWithName(const uint16_t &iBitmap, const std::string & strPartName );
 	bool 	mountStatPartition(string volName);
 	bool 	unmountStatPartition(string volName);
#ifdef SYSTEM_TYPE_LINUX
	bool 	udevhalInstall();
	bool 	udevhalUninstall();
	bool	CopyFile(const string &strFileName, const string &strFilePath);  
	bool	RestartUdevRules();
	bool	StartHalDaemon();
	bool 	StopHalDaemon();
	bool	IsHalDaemonRunning();
	void	CleanMountedPartition();

#endif
 	bool 	mountDynPartition();
 	bool 	unmountDynPartition();
	bool	VerifyCopyFiles(void);
	maserSupport_t	GetMaserType(void);
	uint16_t getPartIndex(string PartName);
	bool SecureMaserCopy(sessionHandle myHdl,string cfgHash, uint16_t reserveByte=0x0000);
	void storeErr(const uint8_t err) { errorInfo.lastErr = err; };
	void storeErrorInfo(uint8_t err);
	void SplitFile(string FileName,size_t BufSize, string dest );
	uint16_t swapBytes(uint16_t Val);
	uint8_t	IPMIRequest(uint8_t subCmd,void *IPMIReqBuf,uint16_t reqBufSz,uint16_t rspBufSz );
	bool PollMaserCmdComplete(int PollCount,uint8_t PollCmd);
	friend class CTestSecureMaser;
	secureMaser(const secureMaser&);
	secureMaser& operator=(const secureMaser&);
	bool system_specific_list_dir(string dir, vector<string>& file_list);
	int isDepCheckSupported(void);
	int isMonolithic(void);
	
	string getLCLXml(const string & cid ,const string &strDevices);
	bool beginSecureUpdate(uint32_t payloadSize);	
	bool processSecureUpdate(string cfgHash);
	bool endSecureUpdate();
	bool populateDynPart(ostringstream *xmlStr);
	void freeResponsePtr(void *pResponse);
	bool byPassRecoveryState();
	string getDevicePath(string& strVolName);
	bool isSGDriverLoaded();
	 
#ifdef SYSTEM_TYPE_WINNT
	void DisableAutoPlay();
	void EnableAutoPlay();
	void SaveAutoPlay();
	void RegBackup();
	void RegRestoreBackup();
	LONG RegCopyKey(HKEY SrcKey, HKEY TrgKey,char* TrgSubKeyName);
	void SetErrorModePopup();
	void ResetErrorModePopup();

public:
	static BOOL DeleteDellTempDirPath();
	static std::string GetDellTempDirPath();

	static std::string GetNewPath(const std::string & sOriginal_Path);
	static std::string GetTempDirPath();
	static std::string GetFileNameFromPath(const std::string &  sFilePath);
	static std::string ReplaceForwardWithBackwordSlash(std::string sOriginalPath);

	static BOOL DeleteDirectoryRecursive(const std::string sDelDirPath);

#endif
	bool compareVendorVersion(const string & mount_point, bool & result);
	bool getVendorVersion(const string & packageXMLPath, string & version);
	bool IsFreshInstall(const string & mount_point, vector<string> & file_list, bool & isFreshInst);
	bool MakeFreshInstall();
	bool ReplaceCurrentFolder(string & mount_point);

};			 
#endif

